@@ -5,7 +5,7 @@ use bevy::{
     ecs::{bundle::Bundle, prelude::*},
     input::{mouse::MouseMotion, prelude::*},
     math::prelude::*,
-    render::prelude::*,
+    render::{camera::Camera3d, prelude::*},
     transform::components::Transform,
 };
 use serde::{Deserialize, Serialize};
@@ -25,9 +25,7 @@ impl FpsCameraPlugin {
 
 impl Plugin for FpsCameraPlugin {
     fn build(&self, app: &mut App) {
-        let app = app
-            .add_system(control_system)
-            .add_event::<ControlEvent>();
+        let app = app.add_system(control_system).add_event::<ControlEvent>();
         if !self.override_input_system {
             app.add_system(default_input_map);
         }
@@ -40,13 +38,13 @@ pub struct FpsCameraBundle {
     #[bundle]
     look_transform: LookTransformBundle,
     #[bundle]
-    perspective: PerspectiveCameraBundle,
+    perspective: PerspectiveCameraBundle<Camera3d>,
 }
 
 impl FpsCameraBundle {
     pub fn new(
         controller: FpsCameraController,
-        mut perspective: PerspectiveCameraBundle,
+        mut perspective: PerspectiveCameraBundle<Camera3d>,
         eye: Vec3,
         target: Vec3,
     ) -> Self {
@@ -96,22 +94,19 @@ pub fn default_input_map(
     controllers: Query<&FpsCameraController>,
 ) {
     // Can only control one camera at a time.
-    let controller = if let Some(controller) = controllers.iter().next() {
+    let controller = if let Some(controller) = controllers.iter().find(|c| {
+        c.enabled
+    }) {
         controller
     } else {
         return;
     };
     let FpsCameraController {
-        enabled,
         translate_sensitivity,
         mouse_rotate_sensitivity,
         ..
     } = *controller;
 
-    if !enabled {
-        return;
-    }
-
     let mut cursor_delta = Vec2::ZERO;
     for event in mouse_motion_events.iter() {
         cursor_delta += event.delta;
@@ -143,14 +138,15 @@ pub fn control_system(
     mut cameras: Query<(&FpsCameraController, &mut LookTransform)>,
 ) {
     // Can only control one camera at a time.
-    let (controller, mut transform) =
-        if let Some((controller, transform)) = cameras.iter_mut().next() {
-            (controller, transform)
+    let mut transform =
+        if let Some((_, transform)) = cameras.iter_mut().find(|c| {
+            c.0.enabled
+        }) {
+            transform
         } else {
             return;
         };
 
-    if controller.enabled {
         let look_vector = transform.look_direction().unwrap();
         let mut look_angles = LookAngles::from_vector(look_vector);
 
@@ -176,7 +172,4 @@ pub fn control_system(
         look_angles.assert_not_looking_up();
 
         transform.target = transform.eye + transform.radius() * look_angles.unit_vector();
-    } else {
-        events.iter(); // Drop the events.
-    }
 }
