@@ -8,7 +8,7 @@ use bevy::{
         prelude::*,
     },
     math::prelude::*,
-    render::prelude::*,
+    render::{camera::Camera3d, prelude::*},
     transform::components::Transform,
 };
 use serde::{Deserialize, Serialize};
@@ -28,9 +28,7 @@ impl OrbitCameraPlugin {
 
 impl Plugin for OrbitCameraPlugin {
     fn build(&self, app: &mut App) {
-        let app = app
-            .add_system(control_system)
-            .add_event::<ControlEvent>();
+        let app = app.add_system(control_system).add_event::<ControlEvent>();
         if !self.override_input_system {
             app.add_system(default_input_map);
         }
@@ -43,13 +41,13 @@ pub struct OrbitCameraBundle {
     #[bundle]
     look_transform: LookTransformBundle,
     #[bundle]
-    perspective: PerspectiveCameraBundle,
+    perspective: PerspectiveCameraBundle<Camera3d>,
 }
 
 impl OrbitCameraBundle {
     pub fn new(
         controller: OrbitCameraController,
-        mut perspective: PerspectiveCameraBundle,
+        mut perspective: PerspectiveCameraBundle<Camera3d>,
         eye: Vec3,
         target: Vec3,
     ) -> Self {
@@ -106,13 +104,14 @@ pub fn default_input_map(
     controllers: Query<&OrbitCameraController>,
 ) {
     // Can only control one camera at a time.
-    let controller = if let Some(controller) = controllers.iter().next() {
+    let controller = if let Some(controller) = controllers.iter().find(|c| {
+        c.enabled
+    }) {
         controller
     } else {
         return;
     };
     let OrbitCameraController {
-        enabled,
         mouse_rotate_sensitivity,
         mouse_translate_sensitivity,
         mouse_wheel_zoom_sensitivity,
@@ -120,10 +119,6 @@ pub fn default_input_map(
         ..
     } = *controller;
 
-    if !enabled {
-        return;
-    }
-
     let mut cursor_delta = Vec2::ZERO;
     for event in mouse_motion_events.iter() {
         cursor_delta += event.delta;
@@ -156,14 +151,15 @@ pub fn control_system(
     mut cameras: Query<(&OrbitCameraController, &mut LookTransform, &Transform)>,
 ) {
     // Can only control one camera at a time.
-    let (controller, mut transform, scene_transform) =
-        if let Some((controller, transform, scene_transform)) = cameras.iter_mut().next() {
-            (controller, transform, scene_transform)
+    let (mut transform, scene_transform) =
+        if let Some((_, transform, scene_transform)) = cameras.iter_mut().find(|c| {
+            c.0.enabled
+        }) {
+            (transform, scene_transform)
         } else {
             return;
         };
 
-    if controller.enabled {
         let mut look_angles = LookAngles::from_vector(-transform.look_direction().unwrap());
         let mut radius_scalar = 1.0;
 
@@ -190,7 +186,4 @@ pub fn control_system(
             .min(1000000.0)
             .max(0.001);
         transform.eye = transform.target + new_radius * look_angles.unit_vector();
-    } else {
-        events.iter(); // Drop the events.
-    }
 }
